package com.fawry.fawry_sdk

import android.app.Activity
import android.util.Log
import androidx.annotation.NonNull
import com.fawry.fawrypay.FawrySdk
import com.fawry.fawrypay.interfaces.FawryPreLaunch
import com.fawry.fawrypay.interfaces.FawrySdkCallbacks
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

class FawrySdkPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    private lateinit var channel: MethodChannel
    private var eventSink: EventSink? = null
    private lateinit var activity: Activity

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, Constants.CHANNEL_NAME)
        channel.setMethodCallHandler(this)

        EventChannel(flutterPluginBinding.binaryMessenger, Constants.EVENT_CHANNEL_NAME)
            .setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(listener: Any?, event: EventSink) {
                    eventSink = event
                }

                override fun onCancel(listener: Any?) {
                    eventSink = null
                }
            })
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            Constants.METHOD_PAY -> startPayment(call, result)
            Constants.METHOD_CARDS -> showCardsManager(call, result)
            else -> result.notImplemented()
        }
    }

    private fun showCardsManager(call: MethodCall, result: Result) {
        val launchJsonString = call.argument<String>(Constants.LAUNCH_MODEL_JSON_ARG)
        val baseUrl = call.argument<String>(Constants.BASE_URL_ARG)
        val lang: FawrySdk.Languages = if (call.argument<String>(Constants.LANGUAGE_ARG) == Constants.LANGUAGE_ARABIC)
            FawrySdk.Languages.ARABIC
        else
            FawrySdk.Languages.ENGLISH
        val launchModel = launchJsonString?.toFawryLaunchModel()

        if (baseUrl.isNullOrBlank()) {
            eventSink?.success(ResponseStatus(Constants.RESPONSE_ERROR, Constants.INVALID_BASE_URL).toJson())
            return
        }

        if (launchModel == null) {
            eventSink?.success(ResponseStatus(Constants.RESPONSE_ERROR, Constants.INVALID_LAUNCH_MODEL).toJson())
            return
        }

        Log.d("testing launch Model", "$launchModel")

        if (::activity.isInitialized) {
            FawrySdk.launchCardManager(
                activity,
                lang,
                baseUrl,
                launchModel,
                createFawrySdkCallbacks()
            )
        }
    }

    private fun startPayment(call: MethodCall, result: Result) {
        val launchJsonString = call.argument<String>(Constants.LAUNCH_MODEL_JSON_ARG)
        val baseUrl = call.argument<String>(Constants.BASE_URL_ARG)
        val lang: FawrySdk.Languages = if (call.argument<String>(Constants.LANGUAGE_ARG) == Constants.LANGUAGE_ARABIC)
            FawrySdk.Languages.ARABIC
        else
            FawrySdk.Languages.ENGLISH
        val launchModel = launchJsonString?.toFawryLaunchModel()

        if (baseUrl.isNullOrBlank()) {
            eventSink?.success(ResponseStatus(Constants.RESPONSE_ERROR, Constants.INVALID_BASE_URL).toJson())
            return
        }

        if (launchModel == null) {
            eventSink?.success(ResponseStatus(Constants.RESPONSE_ERROR, Constants.INVALID_LAUNCH_MODEL).toJson())
            return
        }

        if (::activity.isInitialized) {
            FawrySdk.launchAnonymousSDK(
                activity,
                lang,
                baseUrl,
                launchModel,
                createFawrySdkCallbacks()
            )
        }
    }

    private fun createFawrySdkCallbacks() = object : FawrySdkCallbacks {
        override fun onPreLaunch(onPreLaunch: FawryPreLaunch) {
            onPreLaunch.onContinue()
        }

        override fun onInit() {}

        override fun onSuccess(msg: String, data: Any?) {
            eventSink?.success(
                ResponseStatus(
                    Constants.RESPONSE_SUCCESS,
                    Constants.PAYMENT_SUCCEEDED,
                    data.toString()
                ).toJson()
            )
        }

        override fun onFailure(error: String) {
            eventSink?.success(
                ResponseStatus(
                    Constants.RESPONSE_ERROR,
                    error,
                    null
                ).toJson()
            )
            eventSink?.error(Constants.CODE_PAYMENT_FAILED, error, error)
        }

        override fun onPaymentCompleted(msg: String, data: Any?) {
            eventSink?.success(
                ResponseStatus(
                    Constants.RESPONSE_PAYMENT_COMPLETED,
                    msg,
                    data.toString()
                ).toJson()
            )
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

    override fun onDetachedFromActivity() {}
}
