// ignore_for_file: constant_identifier_names

import 'dart:async';
import 'package:fawry_sdk/fawry_sdk_platform_interface.dart';
import 'package:fawry_sdk/model/fawry_launch_model.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

/// A class providing methods to interact with the Fawry SDK.
class FawrySDK extends FawrySdkPlatformInterface {
  FawrySDK._();

  /// The instance of [FawrySDK].
  static FawrySDK instance = FawrySDK._();

  /// Error code indicating payment failure.
  static const String _CODE_PAYMENT_FAILED = "1";

  /// The name of the event channel for callback results.
  static const String EVENT_CHANNEL_NAME = "event.channel/fawry_sdk";

  /// Response code indicating success.
  static const String RESPONSE_SUCCESS = "100";

  /// Response code indicating an error.
  static const String RESPONSE_ERROR = "101";

  /// Response code indicating payment completion.
  static const String RESPONSE_PAYMENT_COMPLETED = "102";

  /// Language option: Arabic.
  static const String LANGUAGE_ARABIC = "ARABIC";

  /// Language option: English.
  static const String LANGUAGE_ENGLISH = "ENGLISH";

  /// Event channel for callback results.
  final EventChannel _eventChannelCallbackResult =
      const EventChannel(EVENT_CHANNEL_NAME);

  @override
  Future<bool> startPayment({
    required FawryLaunchModel launchModel,
    required String baseURL,
    required String lang,
  }) async {
    try {
      if (!baseURL.endsWith("/")) {
        baseURL = "$baseURL/";
        debugPrint("Fixed Invalid URL");
      }

      return await FawrySdkPlatformInterface.instance.startPayment(
        launchModel: launchModel,
        baseURL: baseURL,
        lang: lang,
      );
    } on PlatformException catch (e) {
      if (e.code == _CODE_PAYMENT_FAILED) {
        throw "Payment Failed: $_CODE_PAYMENT_FAILED. Message: ${e.message}.";
      }
      throw "Unknown Error: Error ->  ${e.message}";
    } catch (e) {
      throw "Error Occurred: Message: $e";
    }
  }

  @override
  Future<bool> openCardsManager({
    required FawryLaunchModel launchModel,
    required String baseURL,
    required String lang,
  }) async {
    try {
      if (!baseURL.endsWith("/")) {
        baseURL = "$baseURL/";
        debugPrint("Fixed Invalid URL");
      }
      return await FawrySdkPlatformInterface.instance.openCardsManager(
        launchModel: launchModel,
        baseURL: baseURL,
        lang: lang,
      );
    } on PlatformException catch (e) {
      if (e.code == _CODE_PAYMENT_FAILED) {
        throw "Payment Failed: $_CODE_PAYMENT_FAILED. Message: ${e.message}.";
      }
      throw "Unknown Error: Error ->  ${e.message}";
    } catch (e) {
      throw "Error Occurred: Message: $e";
    }
  }

  /// Returns a stream of callback results.
  Stream callbackResultStream() {
    return _eventChannelCallbackResult.receiveBroadcastStream();
  }
}
