// ignore_for_file: constant_identifier_names

import 'dart:async';
import 'dart:convert';
import 'package:fawry_sdk/model/fawry_launch_model.dart';
import 'package:flutter/services.dart';
import 'fawry_sdk_platform_interface.dart';

/// A class providing methods to interact with Fawry payment using method channels.
class MethodChannelFlutterFawryPay extends FawrySdkPlatformInterface {
  /// Channel name for communication with native code
  static const String _CHANNEL_NAME = "channel/fawry_sdk";

  /// Method names for communication with native code
  static const String _METHOD_PAY = "fawryStartPayment";
  static const String _METHOD_CARDS = "fawryCardManager";

  /// Error codes
  static const String _ERROR_INIT = "1";

  /// Method channel instance for communication
  static const MethodChannel _channel = MethodChannel(_CHANNEL_NAME);

  @override
  Future<bool> startPayment({
    required FawryLaunchModel launchModel,
    required String baseURL,
    required String lang,
  }) async {
    try {
      // Encode launchModel to JSON and invoke native method
      return await _channel.invokeMethod(_METHOD_PAY, <String, String>{
        'launchModelJson': json.encode(launchModel),
        'baseURL': baseURL,
        'lang': lang
      });
    } on PlatformException catch (e) {
      // Handle specific platform exceptions
      if (e.code == _ERROR_INIT) {
        throw "Error Occurred: Code: $_ERROR_INIT. Message: ${e.message}. Details: SDK Init Error";
      }
      throw "Error Occurred: Code: ${e.code}. Message: ${e.message}. Details: ${e.details}";
    } catch (e) {
      // Catch any other exceptions
      throw "Error Occurred: Message: $e";
    }
  }

  @override
  Future<bool> openCardsManager({
    required FawryLaunchModel launchModel,
    required String baseURL,
    required String lang,
  }) async {
    try {
      // Encode launchModel to JSON and invoke native method
      return await _channel.invokeMethod(_METHOD_CARDS, <String, String>{
        'launchModelJson': json.encode(launchModel),
        'baseURL': baseURL,
        'lang': lang
      });
    } on PlatformException catch (e) {
      // Handle specific platform exceptions
      if (e.code == _ERROR_INIT) {
        throw "Error Occurred: Code: $_ERROR_INIT. Message: ${e.message}. Details: SDK Init Error";
      }
      throw "Error Occurred: Code: ${e.code}. Message: ${e.message}. Details: ${e.details}";
    } catch (e) {
      // Catch any other exceptions
      throw "Error Occurred: Message: $e";
    }
  }
}
