import 'dart:async';
import 'package:fawry_sdk/model/fawry_launch_model.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'fawry_sdk_method_channel.dart';

/// An abstract class defining the platform interface for Fawry SDK.
abstract class FawrySdkPlatformInterface extends PlatformInterface {
  FawrySdkPlatformInterface() : super(token: _token);

  /// Unique token for this platform interface
  static final Object _token = Object();

  /// Instance of MethodChannel implementation
  static FawrySdkPlatformInterface _instance = MethodChannelFlutterFawryPay();

  /// Getter for the instance of this platform interface
  static FawrySdkPlatformInterface get instance => _instance;

  /// Setter for the instance of this platform interface
  static set instance(FawrySdkPlatformInterface instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// Initiates the Fawry payment process.
  Future<bool> startPayment({
    required FawryLaunchModel launchModel,
    required String baseURL,
    required String lang,
  }) {
    throw UnimplementedError('startPayment() has not been implemented.');
  }

  /// Opens the card manager for Fawry payments.
  Future<bool> openCardsManager({
    required FawryLaunchModel launchModel,
    required String baseURL,
    required String lang,
  }) {
    throw UnimplementedError('openCardsManager() has not been implemented.');
  }
}
