import 'package:fawry_sdk/model/payment_methods.dart';
import 'bill_item.dart';
import 'launch_customer_model.dart';
import 'launch_merchant_model.dart';

/// A model representing data required for launching a Fawry payment.
class FawryLaunchModel {
  /// Indicates if 3D Payment is allowed.
  bool allow3DPayment;

  /// List of bill items to be charged.
  List<BillItem> chargeItems;

  /// Launch customer model information (optional).
  LaunchCustomerModel? launchCustomerModel;

  /// Launch merchant model information.
  LaunchMerchantModel launchMerchantModel;

  /// Indicates if login should be skipped.
  bool skipLogin;

  /// Indicates if receipt should be skipped.
  bool skipReceipt;

  /// Indicates if payment should be made with a card token (optional).
  bool? payWithCardToken;

  /// The available payment methods (optional).
  PaymentMethods? paymentMethods;

  /// Constructs a [FawryLaunchModel] with required parameters.
  FawryLaunchModel({
    required this.allow3DPayment,
    required this.chargeItems,
    this.launchCustomerModel,
    required this.launchMerchantModel,
    required this.skipLogin,
    required this.skipReceipt,
    this.payWithCardToken,
    this.paymentMethods,
  });

  /// Converts this [FawryLaunchModel] to a JSON format.
  Map<String, dynamic> toJson() {
    List<Map<String, dynamic>> chargeItemsJson =
        chargeItems.map((item) => item.toJson()).toList();

    Map<String, dynamic>? launchCustomerJson = launchCustomerModel?.toJson();

    Map<String, dynamic> launchMerchantJson = launchMerchantModel.toJson();

    return {
      'allow3DPayment': allow3DPayment,
      'chargeItems': chargeItemsJson,
      'launchCustomerModel': launchCustomerJson,
      'launchMerchantModel': launchMerchantJson,
      'skipLogin': skipLogin,
      'skipReceipt': skipReceipt,
      'merchant': launchMerchantJson,
      'customer': launchCustomerJson,
      'skipCustomerInput': skipLogin,
      'allowVoucher': false,
      'payWithCardToken': payWithCardToken,
      'paymentWithCardToken': payWithCardToken,
      'paymentMethods': paymentMethods?.toShortString(),
      'paymentMethod': paymentMethods?.toShortString(),
    };
  }
}
