import 'dart:io' show Platform;

// ignore: constant_identifier_names
enum PaymentMethods { ALL, CREDIT_CARD, FAWRY_PAY, WALLET }

extension ParseToString on PaymentMethods? {
  /// Converts the [PaymentMethods] enum to a short string representation.
  String toShortString() {
    String? paymentMethod = "ALL";
    if (this != null) {
      paymentMethod = toString().split('.').last;
    }

    return getPlatformPaymentMethod(paymentMethod);
  }

  /// Gets the platform-specific payment method based on the provided [method].
  ///
  /// On iOS, certain payment methods are mapped to specific strings for use
  /// in platform-specific functionality.
  String getPlatformPaymentMethod(String method) {
    if (Platform.isIOS) {
      switch (method) {
        case "CREDIT_CARD":
          return "PayUsingCC";
        case "FAWRY_PAY":
          return "PayAtFawry";
        case "WALLET":
          return "MWALLET";
        default:
          return "all";
      }
    } else {
      return method;
    }
  }
}
